"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var view_1 = require("tns-core-modules/ui/core/view");
var common = require("./accordion.common");
var accordion_common_1 = require("./accordion.common");
var color_1 = require("tns-core-modules/color");
var utils = require("tns-core-modules/utils/utils");
var profiling_1 = require("tns-core-modules/profiling");
var observable_1 = require("tns-core-modules/data/observable");
var proxy_view_container_1 = require("tns-core-modules/ui/proxy-view-container");
var stack_layout_1 = require("tns-core-modules/ui/layouts/stack-layout");
exports.ITEMTAP = 'itemTap';
var NG_VIEW = '_ngViewRef';
global.moduleMerge(common, exports);
var DEFAULT_HEIGHT = 44;
var infinity = utils.layout.makeMeasureSpec(0, utils.layout.UNSPECIFIED);
var majorVersion = utils.ios.MajorVersion;
function notifyForItemAtIndex(owner, nativeView, view, eventName, indexPath) {
    var args = {
        eventName: eventName,
        object: owner,
        index: indexPath.section,
        childIndex: indexPath.row,
        view: view,
        ios: nativeView,
        android: undefined
    };
    owner.notify(args);
    return args;
}
function notifyForHeaderOrFooterAtIndex(owner, nativeView, view, eventName, parentIndex) {
    var args = {
        eventName: eventName,
        object: owner,
        index: parentIndex,
        view: view,
        ios: nativeView,
        android: undefined
    };
    owner.notify(args);
    return args;
}
var DefaultAccordionHeaderViewHeight = 44.0;
var Accordion = (function (_super) {
    __extends(Accordion, _super);
    function Accordion() {
        var _this = _super.call(this) || this;
        _this._itemContentHeights = [];
        _this._itemHeaderHeights = [];
        _this._headerHeights = [];
        _this._footerHeights = [];
        _this._map = new Map();
        _this._mapItemContent = new Map();
        _this._mapItemHeader = new Map();
        _this._mapHeader = new Map();
        _this._mapFooter = new Map();
        return _this;
    }
    Accordion.prototype.updateNativeItems = function (oldItems, newItems) {
        this._ios.reloadData();
    };
    Accordion.prototype.createNativeView = function () {
        return UITableView.new();
    };
    Accordion.prototype.initNativeView = function () {
        _super.prototype.initNativeView.call(this);
        var nativeView = this.nativeViewProtected;
        nativeView.registerClassForCellReuseIdentifier(AccordionCell.class(), this._defaultItemContentTemplate.key);
        nativeView.registerClassForCellReuseIdentifier(AccordionHeaderCell.class(), this._defaultHeaderTemplate.key);
        nativeView.registerClassForCellReuseIdentifier(AccordionFooterCell.class(), this._defaultFooterTemplate.key);
        nativeView.registerClassForHeaderFooterViewReuseIdentifier(AccordionHeader.class(), this._defaultItemHeaderTemplate.key);
        nativeView.estimatedRowHeight = DEFAULT_HEIGHT;
        nativeView.rowHeight = UITableViewAutomaticDimension;
        nativeView.sectionHeaderHeight = UITableViewAutomaticDimension;
        this.ios.separatorColor = UIColor.clearColor;
        if (this.separatorColor) {
            this.ios.separatorColor = new color_1.Color(this.separatorColor).ios;
        }
        this._dataSource = AccordionDataSource.initWithOwner(new WeakRef(this));
        this._delegate = UITableViewDelegateImpl.initWithOwner(new WeakRef(this));
        this._setNativeClipToBounds();
        this._expandedViews = new Map();
        this._indexSet = NSMutableIndexSet.alloc().init();
    };
    Accordion.prototype.onLoaded = function () {
        _super.prototype.onLoaded.call(this);
        if (this._isDataDirty) {
            this.requestLayout();
            this.refresh();
        }
        this.ios.dataSource = this._dataSource;
        this.ios.delegate = this._delegate;
    };
    Accordion.prototype.onUnloaded = function () {
        this.ios.delegate = null;
        _super.prototype.onUnloaded.call(this);
    };
    Accordion.prototype.getItemHeaderHeight = function (index) {
        return this._itemHeaderHeights[index];
    };
    Accordion.prototype.setItemHeaderHeight = function (index, value) {
        this._itemHeaderHeights[index] = value;
    };
    Accordion.prototype.getItemContentHeight = function (index) {
        return this._itemContentHeights[index];
    };
    Accordion.prototype.setItemContentHeight = function (index, value) {
        this._itemContentHeights[index] = value;
    };
    Accordion.prototype.getHeaderHeight = function (index) {
        return this._headerHeights[index];
    };
    Accordion.prototype.setHeaderHeight = function (index, value) {
        this._headerHeights[index] = value;
    };
    Accordion.prototype.getFooterHeight = function (index) {
        return this._footerHeights[index];
    };
    Accordion.prototype.setFooterHeight = function (index, value) {
        this._footerHeights[index] = value;
    };
    Accordion.prototype.disposeNativeView = function () {
        this.ios.delegate = null;
        if (this._indexSet) {
            this._indexSet.removeAllIndexes();
        }
    };
    Accordion.prototype._setNativeClipToBounds = function () {
        this.ios.clipsToBounds = true;
    };
    Object.defineProperty(Accordion.prototype, "ios", {
        get: function () {
            return this.nativeViewProtected;
        },
        enumerable: true,
        configurable: true
    });
    Accordion.prototype.refresh = function () {
        this._map.forEach(function (view, nativeView, map) {
            if (!(view.bindingContext instanceof observable_1.Observable)) {
                view.bindingContext = null;
            }
        });
        this._mapItemContent.forEach(function (view, nativeView, map) {
            if (!(view.bindingContext instanceof observable_1.Observable)) {
                view.bindingContext = null;
            }
        });
        this._mapItemHeader.forEach(function (view, nativeView, map) {
            if (!(view.bindingContext instanceof observable_1.Observable)) {
                view.bindingContext = null;
            }
        });
        this._mapHeader.forEach(function (view, nativeView, map) {
            if (!(view.bindingContext instanceof observable_1.Observable)) {
                view.bindingContext = null;
            }
        });
        this._mapFooter.forEach(function (view, nativeView, map) {
            if (!(view.bindingContext instanceof observable_1.Observable)) {
                view.bindingContext = null;
            }
        });
        if (this.isLoaded) {
            this.ios.reloadData();
            this.requestLayout();
            this._isDataDirty = false;
        }
        else {
            this._isDataDirty = true;
        }
    };
    Accordion.prototype._selectedIndexesUpdatedFromNative = function (newIndexes) {
        accordion_common_1.selectedIndexesProperty.nativeValueChange(this, newIndexes);
    };
    Accordion.prototype.scrollToIndex = function (index) {
        if (this.ios) {
            this.ios.scrollToRowAtIndexPathAtScrollPositionAnimated(NSIndexPath.indexPathForItemInSection(index, 0), 1, false);
        }
    };
    Accordion.prototype.requestLayout = function () {
        if (!this._preparingCell) {
            _super.prototype.requestLayout.call(this);
        }
    };
    Accordion.prototype.measure = function (widthMeasureSpec, heightMeasureSpec) {
        this.widthMeasureSpec = widthMeasureSpec;
        var changed = this._setCurrentMeasureSpecs(widthMeasureSpec, heightMeasureSpec);
        _super.prototype.measure.call(this, widthMeasureSpec, heightMeasureSpec);
        if (changed) {
            this.ios.reloadData();
        }
    };
    Accordion.prototype.onLayout = function (left, top, right, bottom) {
        var _this = this;
        _super.prototype.onLayout.call(this, left, top, right, bottom);
        this._map.forEach(function (childView, accordionCell) {
            var rowHeight = _this._effectiveRowHeight;
            var cellHeight = rowHeight > 0 ? rowHeight : _this.getItemContentHeight(parseInt("" + (childView._accordionItemIndex + 1) + childView._accordionChildItemIndex));
            if (cellHeight) {
                var width = view_1.layout.getMeasureSpecSize(_this.widthMeasureSpec);
                view_1.View.layoutChild(_this, childView, 0, 0, width, cellHeight);
            }
        });
        this._mapItemHeader.forEach(function (childView, accordionHeader) {
            var rowHeight = _this._effectiveRowHeight;
            var cellHeight = rowHeight > 0 ? rowHeight : _this.getItemHeaderHeight(childView._accordionItemIndex);
            if (cellHeight) {
                var width = view_1.layout.getMeasureSpecSize(_this.widthMeasureSpec);
                view_1.View.layoutChild(_this, childView, 0, 0, width, cellHeight);
            }
        });
        this._mapHeader.forEach(function (childView, accordionHeaderCell) {
            var rowHeight = _this._effectiveRowHeight;
            var cellHeight = rowHeight > 0 ? rowHeight : _this.getItemHeaderHeight(childView._accordionItemIndex);
            if (cellHeight) {
                var width = view_1.layout.getMeasureSpecSize(_this.widthMeasureSpec);
                view_1.View.layoutChild(_this, childView, 0, 0, width, cellHeight);
            }
        });
        this._mapFooter.forEach(function (childView, accordionFooterCell) {
            var rowHeight = _this._effectiveRowHeight;
            var cellHeight = rowHeight > 0 ? rowHeight : _this.getItemHeaderHeight(childView._accordionItemIndex);
            if (cellHeight) {
                var width = view_1.layout.getMeasureSpecSize(_this.widthMeasureSpec);
                view_1.View.layoutChild(_this, childView, 0, 0, width, cellHeight);
            }
        });
    };
    Accordion.prototype.onMeasure = function (widthMeasureSpec, heightMeasureSpec) {
        var _this = this;
        _super.prototype.onMeasure.call(this, widthMeasureSpec, heightMeasureSpec);
        this._map.forEach(function (childView, accordionCell) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
        this._mapItemHeader.forEach(function (childView, accordionHeader) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
        this._mapItemContent.forEach(function (childView, accordionCell) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
        this._mapHeader.forEach(function (childView, accordionHeaderCell) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
        this._mapFooter.forEach(function (childView, accordionFooterCell) {
            view_1.View.measureChild(_this, childView, childView._currentWidthMeasureSpec, childView._currentHeightMeasureSpec);
        });
    };
    Accordion.prototype._layoutHeaderCell = function (cellView, indexPath) {
        if (cellView) {
            var rowHeight = this._effectiveRowHeight;
            var heightMeasureSpec = rowHeight >= 0 ? view_1.layout.makeMeasureSpec(rowHeight, view_1.layout.EXACTLY) : infinity;
            var measuredSize = view_1.View.measureChild(this, cellView, this.widthMeasureSpec, heightMeasureSpec);
            var height = measuredSize.measuredHeight;
            this.setHeaderHeight(indexPath.section, height);
            return height;
        }
        return this.ios.estimatedRowHeight;
    };
    Accordion.prototype._layoutFooterCell = function (cellView, indexPath) {
        if (cellView) {
            var rowHeight = this._effectiveRowHeight;
            var heightMeasureSpec = rowHeight >= 0 ? view_1.layout.makeMeasureSpec(rowHeight, view_1.layout.EXACTLY) : infinity;
            var measuredSize = view_1.View.measureChild(this, cellView, this.widthMeasureSpec, heightMeasureSpec);
            var height = measuredSize.measuredHeight;
            this.setFooterHeight(indexPath.section, height);
            return height;
        }
        return this.ios.estimatedRowHeight;
    };
    Accordion.prototype._layoutCell = function (cellView, indexPath) {
        if (cellView) {
            var rowHeight = this._effectiveRowHeight;
            var heightMeasureSpec = rowHeight >= 0 ? view_1.layout.makeMeasureSpec(rowHeight, view_1.layout.EXACTLY) : infinity;
            var measuredSize = view_1.View.measureChild(this, cellView, this.widthMeasureSpec, heightMeasureSpec);
            var height = measuredSize.measuredHeight;
            this.setItemContentHeight(parseInt("" + (indexPath.section + 1) + indexPath.row), height);
            return height;
        }
        return this.ios.estimatedRowHeight;
    };
    Accordion.prototype._layoutHeader = function (cellView, section) {
        if (cellView) {
            var rowHeight = this._effectiveRowHeight;
            var heightMeasureSpec = rowHeight >= 0 ? view_1.layout.makeMeasureSpec(rowHeight, view_1.layout.EXACTLY) : infinity;
            var measuredSize = view_1.View.measureChild(this, cellView, this.widthMeasureSpec, heightMeasureSpec);
            var height = measuredSize.measuredHeight;
            this.setItemHeaderHeight(section, height);
            return height;
        }
        return this.ios.estimatedSectionHeaderHeight;
    };
    Accordion.prototype._prepareCell = function (cell, indexPath) {
        var cellHeight;
        try {
            this._preparingCell = true;
            var view = cell.view;
            if (!view) {
                view = this._getItemContentTemplate(indexPath.section, indexPath.row).createView();
            }
            var args = {
                eventName: accordion_common_1.AccordionBase.itemContentLoadingEvent,
                object: this,
                index: indexPath.section,
                childIndex: (indexPath.row - (this._getHasHeader() ? 1 : 0)),
                view: view,
                ios: null,
                android: undefined
            };
            this.notify(args);
            view = args.view || this._getDefaultItemContentContent(indexPath.section, indexPath.row);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!cell.view) {
                cell.owner = new WeakRef(view);
            }
            else if (cell.view !== view) {
                this._removeItemContentContainer(cell);
                cell.view.nativeViewProtected.removeFromSuperview();
                cell.owner = new WeakRef(view);
            }
            this._prepareItemContent(view, indexPath.section, indexPath.row);
            view._accordionChildItemIndex = indexPath.row;
            view._accordionItemIndex = indexPath.section;
            this._map.set(cell, view);
            if (view && !view.parent) {
                this._addView(view);
                cell.contentView.addSubview(view.nativeViewProtected);
            }
            cellHeight = this._layoutCell(view, indexPath);
        }
        finally {
            this._preparingCell = false;
        }
        return cellHeight;
    };
    Accordion.prototype._prepareItemHeaderView = function (header, section) {
        var headerHeight;
        try {
            this._preparingCell = true;
            var view = header.view;
            if (!view) {
                view = this._getItemHeaderTemplate(section).createView();
            }
            var args = notifyForHeaderOrFooterAtIndex(this, header, view, accordion_common_1.AccordionBase.itemHeaderLoadingEvent, section);
            view = args.view || this._getDefaultItemHeaderContent(section);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!header.view) {
                header.owner = new WeakRef(view);
            }
            else if (header.view !== view) {
                this._removeItemHeaderContainer(header);
                header.view.nativeViewProtected.removeFromSuperview();
                header.owner = new WeakRef(view);
            }
            this._prepareItemHeader(view, section);
            view._accordionItemIndex = section;
            this._mapItemHeader.set(header, view);
            if (view && !view.parent) {
                this._addView(view);
                header.contentView.addSubview(view.nativeViewProtected);
            }
            headerHeight = this._layoutHeader(view, section);
        }
        finally {
            this._preparingCell = false;
        }
        return headerHeight;
    };
    Accordion.prototype._prepareHeaderCell = function (cell, indexPath) {
        var cellHeight;
        try {
            this._preparingCell = true;
            var view = cell.view;
            if (!view) {
                view = this._getHeaderTemplate(indexPath.section).createView();
            }
            var args = {
                eventName: accordion_common_1.AccordionBase.footerLoadingEvent,
                object: this,
                index: indexPath.section,
                view: view,
                ios: null,
                android: undefined
            };
            this.notify(args);
            view = args.view || this._getDefaultHeaderContent(indexPath.row);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!cell.view) {
                cell.owner = new WeakRef(view);
            }
            else if (cell.view !== view) {
                this._removeHeaderContainer(cell);
                cell.view.nativeViewProtected.removeFromSuperview();
                cell.owner = new WeakRef(view);
            }
            this._prepareItemHeader(view, indexPath.section);
            view._accordionChildItemIndex = indexPath.section;
            this._map.set(cell, view);
            if (view && !view.parent) {
                this._addView(view);
                cell.contentView.addSubview(view.nativeViewProtected);
            }
            cellHeight = this._layoutHeaderCell(view, indexPath);
        }
        finally {
            this._preparingCell = false;
        }
        return cellHeight;
    };
    Accordion.prototype._prepareFooterCell = function (cell, indexPath) {
        var cellHeight;
        try {
            this._preparingCell = true;
            var view = cell.view;
            if (!view) {
                view = this._getFooterTemplate(indexPath.section).createView();
            }
            var args = {
                eventName: accordion_common_1.AccordionBase.headerLoadingEvent,
                object: this,
                index: indexPath.section,
                view: view,
                ios: null,
                android: undefined
            };
            this.notify(args);
            view = args.view || this._getDefaultFooterContent(indexPath.section);
            if (view instanceof proxy_view_container_1.ProxyViewContainer) {
                var sp = new stack_layout_1.StackLayout();
                sp.addChild(view);
                view = sp;
            }
            if (!cell.view) {
                cell.owner = new WeakRef(view);
            }
            else if (cell.view !== view) {
                this._removeFooterContainer(cell);
                cell.view.nativeViewProtected.removeFromSuperview();
                cell.owner = new WeakRef(view);
            }
            this._prepareFooterItem(view, indexPath.section);
            view._accordionChildItemIndex = indexPath.section;
            this._map.set(cell, view);
            if (view && !view.parent) {
                this._addView(view);
                cell.contentView.addSubview(view.nativeViewProtected);
            }
            cellHeight = this._layoutFooterCell(view, indexPath);
        }
        finally {
            this._preparingCell = false;
        }
        return cellHeight;
    };
    Accordion.prototype._removeItemHeaderContainer = function (header) {
        var view = header.view;
        if (!(view.parent instanceof Accordion)) {
            this._removeView(view.parent);
        }
        var preparing = this._preparingCell;
        this._preparingCell = true;
        view.parent._removeView(view);
        view._accordionItemIndex = undefined;
        this._preparingCell = preparing;
        this._mapItemHeader.delete(header);
    };
    Accordion.prototype._removeItemContentContainer = function (cell) {
        var view = cell.view;
        if (!(view.parent instanceof Accordion)) {
            this._removeView(view.parent);
        }
        var preparing = this._preparingCell;
        this._preparingCell = true;
        view.parent._removeView(view);
        view._accordionChildItemIndex = undefined;
        view._accordionItemIndex = undefined;
        this._preparingCell = preparing;
        this._map.delete(cell);
    };
    Accordion.prototype._removeHeaderContainer = function (cell) {
        var view = cell.view;
        if (!(view.parent instanceof Accordion)) {
            this._removeView(view.parent);
        }
        var preparing = this._preparingCell;
        this._preparingCell = true;
        view.parent._removeView(view);
        view._accordionChildItemIndex = undefined;
        this._preparingCell = preparing;
        this._mapHeader.delete(cell);
    };
    Accordion.prototype._removeFooterContainer = function (cell) {
        var view = cell.view;
        if (!(view.parent instanceof Accordion)) {
            this._removeView(view.parent);
        }
        var preparing = this._preparingCell;
        this._preparingCell = true;
        view.parent._removeView(view);
        view._accordionChildItemIndex = undefined;
        this._preparingCell = preparing;
        this._mapFooter.delete(cell);
    };
    Object.defineProperty(Accordion.prototype, "_childrenCount", {
        get: function () {
            return this._map.size;
        },
        enumerable: true,
        configurable: true
    });
    Accordion.prototype.eachChildView = function (callback) {
        this._map.forEach(function (view, key) {
            callback(view);
        });
    };
    Accordion.prototype.updateNativeIndexes = function (oldIndexes, newIndexes) {
        var _this = this;
        var allowMultiple = String(this.allowMultiple) === 'true';
        if (allowMultiple) {
            newIndexes.forEach(function (index) {
                if (!_this._expandedViews.get(index)) {
                    _this._expandedViews.set(index, true);
                    _this._indexSet.addIndex(index);
                }
            });
            this.ios.reloadData();
        }
        else {
            if (newIndexes.length > 0) {
                var index_1 = newIndexes.length - 1;
                var newItems = [index_1];
                this._expandedViews.clear();
                this._indexSet.removeAllIndexes();
                if (!this._expandedViews.get(index_1)) {
                    this._expandedViews.set(index_1, true);
                    this._indexSet.addIndex(index_1);
                    this.ios.reloadData();
                }
            }
        }
    };
    Accordion.prototype.expandAll = function () {
        var length = this.items ? this.items.length : 0;
        var allowMultiple = String(this.allowMultiple) === 'true';
        if (!allowMultiple) {
            this._expandedViews.clear();
            this._indexSet.removeAllIndexes();
            this._expandedViews.set(length - 1, true);
            this._indexSet.addIndex(length - 1);
            this.ios.reloadData();
            return;
        }
        for (var i = 0; i < length; i++) {
            this._expandedViews.set(i, true);
            this._indexSet.addIndex(0);
        }
        this.ios.reloadData();
    };
    Accordion.prototype.collapseAll = function () {
        this._expandedViews.clear();
        this._indexSet.removeAllIndexes();
        this.ios.reloadData();
    };
    Accordion.prototype.collapseItem = function (index) {
        if (this._expandedViews.has(index)) {
            this._expandedViews.delete(index);
            this._indexSet.removeIndex(index);
            this.ios.reloadData();
        }
    };
    Accordion.prototype.expandItem = function (index) {
        var _this = this;
        var reloadSection = function (index) {
            var section = NSMutableIndexSet.alloc().initWithIndex(index);
            _this.ios.reloadSectionsWithRowAnimation(section, 100);
        };
        var removeSection = function (index) {
            var section = NSMutableIndexSet.alloc().initWithIndex(index);
            _this.ios.reloadSectionsWithRowAnimation(section, 4);
        };
        var allowMultiple = String(this.allowMultiple) === 'true';
        if (allowMultiple) {
            if (!this._expandedViews.get(index)) {
                this.itemExpanded(index);
                this._expandedViews.set(index, true);
                this._indexSet.addIndex(index);
            }
            else {
                this._expandedViews.delete(index);
                this._indexSet.removeIndex(index);
                this.itemCollapsed(index);
            }
            reloadSection(index);
            this._selectedIndexesUpdatedFromNative(Array.from(this._expandedViews.keys()));
        }
        else {
            if (this._expandedViews.has(index)) {
                this._expandedViews.delete(index);
                this._indexSet.removeIndex(index);
                this.itemCollapsed(index);
                reloadSection(index);
            }
            else if (this._expandedViews.size > 0) {
                var old = this._expandedViews.keys().next().value;
                this._expandedViews.delete(old);
                this._indexSet.removeIndex(old);
                reloadSection(old);
                this.itemCollapsed(old);
                this._expandedViews.set(index, true);
                this._indexSet.addIndex(index);
                reloadSection(index);
                this.itemExpanded(index);
            }
            else {
                this._expandedViews.set(index, true);
                this._indexSet.addIndex(index);
                this.itemExpanded(index);
                reloadSection(index);
            }
            this._selectedIndexesUpdatedFromNative(Array.from(this._expandedViews.keys()));
        }
    };
    Accordion.prototype.itemIsExpanded = function (index) {
        if (this._expandedViews.has(index)) {
            return this._expandedViews.get(index);
        }
        return false;
    };
    Accordion.prototype.itemExpanded = function (index) {
        this.notify({
            eventName: 'itemExpanded',
            object: observable_1.fromObject({
                value: index
            })
        });
    };
    Accordion.prototype.itemCollapsed = function (index) {
        this.notify({
            eventName: 'itemCollapsed',
            object: observable_1.fromObject({
                value: index
            })
        });
    };
    Accordion.prototype._onItemHeaderRowHeightPropertyChanged = function (oldValue, newValue) {
        var value = view_1.layout.toDeviceIndependentPixels(this._effectiveRowHeight);
        var nativeView = this.ios;
        if (value < 0) {
            nativeView.sectionHeaderHeight = UITableViewAutomaticDimension;
            nativeView.estimatedSectionHeaderHeight = DEFAULT_HEIGHT;
            this._delegate = UITableViewDelegateImpl.initWithOwner(new WeakRef(this));
        }
        else {
            nativeView.sectionHeaderHeight = value;
            nativeView.estimatedSectionHeaderHeight = value;
            this._delegate = UITableViewRowHeightDelegateImpl.initWithOwner(new WeakRef(this));
        }
        if (this.isLoaded) {
            nativeView.delegate = this._delegate;
        }
        _super.prototype._onItemHeaderRowHeightPropertyChanged.call(this, oldValue, newValue);
    };
    Accordion.prototype._onItemContentRowHeightPropertyChanged = function (oldValue, newValue) {
        var value = view_1.layout.toDeviceIndependentPixels(this._effectiveRowHeight);
        var nativeView = this.ios;
        if (value < 0) {
            nativeView.rowHeight = UITableViewAutomaticDimension;
            nativeView.estimatedRowHeight = DEFAULT_HEIGHT;
            this._delegate = UITableViewDelegateImpl.initWithOwner(new WeakRef(this));
        }
        else {
            nativeView.rowHeight = value;
            nativeView.estimatedRowHeight = value;
            this._delegate = UITableViewRowHeightDelegateImpl.initWithOwner(new WeakRef(this));
        }
        if (this.isLoaded) {
            nativeView.delegate = this._delegate;
        }
        _super.prototype._onItemContentRowHeightPropertyChanged.call(this, oldValue, newValue);
    };
    Accordion.prototype[accordion_common_1.itemHeaderTemplatesProperty.getDefault] = function () {
        return null;
    };
    Accordion.prototype[accordion_common_1.itemHeaderTemplatesProperty.setNative] = function (value) {
        this._itemHeaderTemplatesInternal = new Array(this._defaultItemHeaderTemplate);
        if (value) {
            for (var i = 0, length_1 = value.length; i < length_1; i++) {
                this.ios.registerClassForHeaderFooterViewReuseIdentifier(AccordionHeader.class(), "item-header-" + value[i].key);
            }
            this._itemHeaderTemplatesInternal = this._itemHeaderTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    Accordion.prototype[accordion_common_1.itemContentTemplatesProperty.getDefault] = function () {
        return null;
    };
    Accordion.prototype[accordion_common_1.itemContentTemplatesProperty.setNative] = function (value) {
        this._itemContentTemplatesInternal = new Array(this._defaultItemContentTemplate);
        if (value) {
            for (var i = 0, length_2 = value.length; i < length_2; i++) {
                this.ios.registerClassForCellReuseIdentifier(AccordionCell.class(), "item-content-" + value[i].key);
            }
            this._itemContentTemplatesInternal = this._itemContentTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    Accordion.prototype[accordion_common_1.footerTemplatesProperty.getDefault] = function () {
        return null;
    };
    Accordion.prototype[accordion_common_1.footerTemplatesProperty.setNative] = function (value) {
        this._footerTemplatesInternal = new Array(this._defaultFooterTemplate);
        if (value) {
            for (var i = 0, length_3 = value.length; i < length_3; i++) {
                this.ios.registerClassForCellReuseIdentifier(AccordionFooterCell.class(), "footer-" + value[i].key);
            }
            this._footerTemplatesInternal = this._footerTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    Accordion.prototype[accordion_common_1.headerTemplatesProperty.getDefault] = function () {
        return null;
    };
    Accordion.prototype[accordion_common_1.headerTemplatesProperty.setNative] = function (value) {
        this._headerTemplatesInternal = new Array(this._defaultHeaderTemplate);
        if (value) {
            for (var i = 0, length_4 = value.length; i < length_4; i++) {
                this.ios.registerClassForCellReuseIdentifier(AccordionHeaderCell.class(), "header-" + value[i].key);
            }
            this._headerTemplatesInternal = this._headerTemplatesInternal.concat(value);
        }
        this.refresh();
    };
    Accordion.prototype[accordion_common_1.iosEstimatedItemHeaderRowHeightProperty.getDefault] = function () {
        return DEFAULT_HEIGHT;
    };
    Accordion.prototype[accordion_common_1.iosEstimatedItemHeaderRowHeightProperty.setNative] = function (value) {
        var nativeView = this.ios;
        var estimatedHeight = view_1.Length.toDevicePixels(value, 0);
        nativeView.estimatedSectionHeaderHeight = estimatedHeight < 0 ? DEFAULT_HEIGHT : estimatedHeight;
    };
    Accordion.prototype[accordion_common_1.iosEstimatedItemContentRowHeightProperty.getDefault] = function () {
        return DEFAULT_HEIGHT;
    };
    Accordion.prototype[accordion_common_1.iosEstimatedItemContentRowHeightProperty.setNative] = function (value) {
        var nativeView = this.ios;
        var estimatedHeight = view_1.Length.toDevicePixels(value, 0);
        nativeView.estimatedRowHeight = estimatedHeight < 0 ? DEFAULT_HEIGHT : estimatedHeight;
    };
    __decorate([
        profiling_1.profile,
        __metadata("design:type", Function),
        __metadata("design:paramtypes", []),
        __metadata("design:returntype", void 0)
    ], Accordion.prototype, "onLoaded", null);
    return Accordion;
}(accordion_common_1.AccordionBase));
exports.Accordion = Accordion;
var AccordionHeaderTap = (function (_super) {
    __extends(AccordionHeaderTap, _super);
    function AccordionHeaderTap() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionHeaderTap.initWithOwner = function (owner) {
        var tap = new AccordionHeaderTap();
        tap._owner = owner;
        return tap;
    };
    AccordionHeaderTap.prototype.tap = function (args) {
        var owner = this._owner.get();
        var current = args.view.tag;
        var data = owner._getParentData(current);
        var _args = {
            eventName: accordion_common_1.AccordionBase.itemHeaderTapEvent,
            data: data,
            object: owner,
            parentIndex: current,
            view: null,
            ios: args.view,
            android: undefined
        };
        owner.notify(_args);
        var reloadSection = function (index) {
            var section = NSMutableIndexSet.alloc().initWithIndex(index);
            owner.ios.reloadSectionsWithRowAnimation(section, 100);
        };
        var removeSection = function (index) {
            var section = NSMutableIndexSet.alloc().initWithIndex(index);
            owner.ios.reloadSectionsWithRowAnimation(section, 4);
        };
        var allowMultiple = String(owner.allowMultiple) === 'true';
        if (allowMultiple) {
            if (!owner._expandedViews.get(current)) {
                owner.itemExpanded(current);
                owner._expandedViews.set(current, true);
                owner._indexSet.addIndex(current);
            }
            else {
                owner._expandedViews.delete(current);
                owner._indexSet.removeIndex(current);
                owner.itemCollapsed(current);
            }
            reloadSection(current);
            owner._selectedIndexesUpdatedFromNative(Array.from(owner._expandedViews.keys()));
        }
        else {
            if (owner._expandedViews.has(current)) {
                owner._expandedViews.delete(current);
                owner._indexSet.removeIndex(current);
                owner.itemCollapsed(current);
                reloadSection(current);
            }
            else if (owner._expandedViews.size > 0) {
                var old = owner._expandedViews.keys().next().value;
                owner._expandedViews.delete(old);
                owner._indexSet.removeIndex(old);
                reloadSection(old);
                owner.itemCollapsed(old);
                owner._expandedViews.set(current, true);
                owner._indexSet.addIndex(current);
                reloadSection(current);
                owner.itemExpanded(current);
            }
            else {
                owner._expandedViews.set(current, true);
                owner._indexSet.addIndex(current);
                owner.itemExpanded(current);
                reloadSection(current);
            }
            owner._selectedIndexesUpdatedFromNative(Array.from(owner._expandedViews.keys()));
        }
    };
    AccordionHeaderTap.ObjCExposedMethods = {
        'tap': { returns: interop.types.void, params: [interop.types.id] }
    };
    return AccordionHeaderTap;
}(NSObject));
var AccordionHeader = (function (_super) {
    __extends(AccordionHeader, _super);
    function AccordionHeader() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionHeader.initWithEmptyBackground = function () {
        var header = AccordionHeader.new();
        header.contentView.backgroundColor = null;
        return header;
    };
    Object.defineProperty(AccordionHeader.prototype, "view", {
        get: function () {
            return this.owner ? this.owner.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    AccordionHeader.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.view ? this.view.parent : null);
        if (parent && !newSuperview) {
            parent._removeItemHeaderContainer(this);
        }
    };
    return AccordionHeader;
}(UITableViewHeaderFooterView));
exports.AccordionHeader = AccordionHeader;
var AccordionCell = (function (_super) {
    __extends(AccordionCell, _super);
    function AccordionCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionCell.initWithEmptyBackground = function () {
        var cell = AccordionCell.new();
        cell.backgroundColor = null;
        return cell;
    };
    AccordionCell.prototype.initWithStyleReuseIdentifier = function (style, reuseIdentifier) {
        var cell = _super.prototype.initWithStyleReuseIdentifier.call(this, style, reuseIdentifier);
        cell.backgroundColor = null;
        return cell;
    };
    AccordionCell.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.view ? this.view.parent : null);
        if (parent && !newSuperview) {
            parent._removeItemContentContainer(this);
        }
    };
    Object.defineProperty(AccordionCell.prototype, "view", {
        get: function () {
            return this.owner ? this.owner.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    return AccordionCell;
}(UITableViewCell));
exports.AccordionCell = AccordionCell;
var AccordionHeaderCell = (function (_super) {
    __extends(AccordionHeaderCell, _super);
    function AccordionHeaderCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionHeaderCell.initWithEmptyBackground = function () {
        var cell = AccordionCell.new();
        cell.backgroundColor = null;
        return cell;
    };
    AccordionHeaderCell.prototype.initWithStyleReuseIdentifier = function (style, reuseIdentifier) {
        var cell = _super.prototype.initWithStyleReuseIdentifier.call(this, style, reuseIdentifier);
        cell.backgroundColor = null;
        return cell;
    };
    AccordionHeaderCell.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.view ? this.view.parent : null);
        if (parent && !newSuperview) {
            parent._removeHeaderContainer(this);
        }
    };
    Object.defineProperty(AccordionHeaderCell.prototype, "view", {
        get: function () {
            return this.owner ? this.owner.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    return AccordionHeaderCell;
}(UITableViewCell));
exports.AccordionHeaderCell = AccordionHeaderCell;
var AccordionFooterCell = (function (_super) {
    __extends(AccordionFooterCell, _super);
    function AccordionFooterCell() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionFooterCell.initWithEmptyBackground = function () {
        var cell = AccordionCell.new();
        cell.backgroundColor = null;
        return cell;
    };
    AccordionFooterCell.prototype.initWithStyleReuseIdentifier = function (style, reuseIdentifier) {
        var cell = _super.prototype.initWithStyleReuseIdentifier.call(this, style, reuseIdentifier);
        cell.backgroundColor = null;
        return cell;
    };
    AccordionFooterCell.prototype.willMoveToSuperview = function (newSuperview) {
        var parent = (this.view ? this.view.parent : null);
        if (parent && !newSuperview) {
            parent._removeFooterContainer(this);
        }
    };
    Object.defineProperty(AccordionFooterCell.prototype, "view", {
        get: function () {
            return this.owner ? this.owner.get() : null;
        },
        enumerable: true,
        configurable: true
    });
    return AccordionFooterCell;
}(UITableViewCell));
exports.AccordionFooterCell = AccordionFooterCell;
var AccordionDataSource = (function (_super) {
    __extends(AccordionDataSource, _super);
    function AccordionDataSource() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    AccordionDataSource.initWithOwner = function (owner) {
        var dataSource = AccordionDataSource.new();
        dataSource._owner = owner;
        return dataSource;
    };
    AccordionDataSource.prototype.tableViewNumberOfRowsInSection = function (tableView, section) {
        var owner = this._owner.get();
        if (owner._expandedViews.has(section) && owner._expandedViews.get(section)) {
            var parentData = owner && owner.items ? owner._getParentData(section) : [];
            return parentData[owner.childItems] ? (parentData[owner.childItems].length + (owner._getHasHeader() ? 1 : 0) + (owner._getHasFooter() ? 1 : 0)) : 0;
        }
        else {
            return 0;
        }
    };
    AccordionDataSource.prototype.numberOfSectionsInTableView = function (tableView) {
        var owner = this._owner.get();
        return (owner && owner.items) ? owner.items.length : 0;
    };
    AccordionDataSource.prototype.tableViewCellForRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        var cell;
        var total = this.tableViewNumberOfRowsInSection(tableView, indexPath.section);
        if (indexPath.row === 0 && owner._getHasHeader()) {
            if (owner) {
                var template = owner._getHeaderTemplate(indexPath.section);
                cell = (tableView.dequeueReusableCellWithIdentifier("header-" + template.key) || AccordionHeaderCell.initWithEmptyBackground());
                owner._prepareHeaderCell(cell, indexPath);
                var cellView = cell.view;
                if (cellView && cellView.isLayoutRequired) {
                    var width = view_1.layout.getMeasureSpecSize(owner.widthMeasureSpec);
                    var rowHeight = owner._effectiveRowHeight;
                    var cellHeight = rowHeight > 0 ? rowHeight : owner.getHeaderHeight(indexPath.row);
                    view_1.View.layoutChild(owner, cellView, 0, 0, width, cellHeight ? cellHeight : 0);
                }
            }
            else {
                cell = AccordionHeaderCell.initWithEmptyBackground();
            }
            return cell;
        }
        if (indexPath.row === total - 1 && owner._getHasFooter) {
            if (owner) {
                var template = owner._getFooterTemplate(indexPath.section);
                cell = (tableView.dequeueReusableCellWithIdentifier("footer-" + template.key) || AccordionFooterCell.initWithEmptyBackground());
                owner._prepareFooterCell(cell, indexPath);
                var cellView = cell.view;
                if (cellView && cellView.isLayoutRequired) {
                    var width = view_1.layout.getMeasureSpecSize(owner.widthMeasureSpec);
                    var rowHeight = owner._effectiveRowHeight;
                    var cellHeight = rowHeight > 0 ? rowHeight : owner.getFooterHeight(indexPath.row);
                    view_1.View.layoutChild(owner, cellView, 0, 0, width, cellHeight ? cellHeight : 0);
                }
            }
            else {
                cell = AccordionFooterCell.initWithEmptyBackground();
            }
            return cell;
        }
        if (owner) {
            var template = owner._getItemContentTemplate(indexPath.section, indexPath.row);
            cell = (tableView.dequeueReusableCellWithIdentifier("item-content-" + template.key) || AccordionCell.initWithEmptyBackground());
            owner._prepareCell(cell, indexPath);
            var cellView = cell.view;
            if (cellView && cellView.isLayoutRequired) {
                var width = view_1.layout.getMeasureSpecSize(owner.widthMeasureSpec);
                var rowHeight = owner._effectiveRowHeight;
                var cellHeight = rowHeight > 0 ? rowHeight : owner.getItemContentHeight(parseInt("" + (indexPath.section + 1) + indexPath.row));
                view_1.View.layoutChild(owner, cellView, 0, 0, width, cellHeight ? cellHeight : 0);
            }
        }
        else {
            cell = AccordionCell.initWithEmptyBackground();
        }
        return cell;
    };
    AccordionDataSource.ObjCProtocols = [UITableViewDataSource];
    return AccordionDataSource;
}(NSObject));
exports.AccordionDataSource = AccordionDataSource;
var UITableViewRowHeightDelegateImpl = (function (_super) {
    __extends(UITableViewRowHeightDelegateImpl, _super);
    function UITableViewRowHeightDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UITableViewRowHeightDelegateImpl.initWithOwner = function (owner) {
        var delegate = UITableViewRowHeightDelegateImpl.new();
        delegate._owner = owner;
        return delegate;
    };
    UITableViewRowHeightDelegateImpl.prototype.tableViewWillDisplayCellForRowAtIndexPath = function (tableView, cell, indexPath) {
        var owner = this._owner.get();
        if (owner && (indexPath.row === owner.items.length - 1)) {
            owner.notify({ eventName: Accordion.loadMoreItemsEvent, object: owner });
        }
    };
    UITableViewRowHeightDelegateImpl.prototype.tableViewWillSelectRowAtIndexPath = function (tableView, indexPath) {
        var cell = tableView.cellForRowAtIndexPath(indexPath);
        var owner = this._owner.get();
        if (owner) {
            notifyForItemAtIndex(owner, cell, cell.view, exports.ITEMTAP, indexPath);
        }
        return indexPath;
    };
    UITableViewRowHeightDelegateImpl.prototype.tableViewDidSelectRowAtIndexPath = function (tableView, indexPath) {
        tableView.deselectRowAtIndexPathAnimated(indexPath, true);
        return indexPath;
    };
    UITableViewRowHeightDelegateImpl.prototype.tableViewHeightForRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        if (!owner) {
            return tableView.estimatedRowHeight;
        }
        return view_1.layout.toDeviceIndependentPixels(owner._effectiveRowHeight);
    };
    UITableViewRowHeightDelegateImpl.prototype.tableViewHeightForHeaderInSection = function (tableView, section) {
        var owner = this._owner.get();
        if (!owner) {
            return tableView.estimatedSectionHeaderHeight;
        }
        return view_1.layout.toDeviceIndependentPixels(owner._effectiveRowHeight);
    };
    UITableViewRowHeightDelegateImpl.ObjCProtocols = [UITableViewDelegate];
    return UITableViewRowHeightDelegateImpl;
}(NSObject));
var UITableViewDelegateImpl = (function (_super) {
    __extends(UITableViewDelegateImpl, _super);
    function UITableViewDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    UITableViewDelegateImpl.initWithOwner = function (owner) {
        var delegate = UITableViewDelegateImpl.new();
        delegate._owner = owner;
        delegate._measureItemHeaderMap = new Map();
        delegate._measureItemContentMap = new Map();
        delegate._measureHeaderMap = new Map();
        delegate._measureFooterMap = new Map();
        return delegate;
    };
    UITableViewDelegateImpl.prototype.tableViewHeightForRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        if (!owner) {
            return tableView.estimatedRowHeight;
        }
        var height;
        var total = tableView.numberOfRowsInSection(indexPath.section);
        if (indexPath.row === 0 && owner._getHasHeader()) {
            height = owner.getHeaderHeight(indexPath.section);
            if (height === undefined) {
                var template = owner._getHeaderTemplate(indexPath.section);
                var cell = this._measureHeaderMap.get(template.key);
                if (!cell) {
                    cell = tableView.dequeueReusableCellWithIdentifier("header-" + template.key) || AccordionHeaderCell.initWithEmptyBackground();
                    this._measureHeaderMap.set(template.key, cell);
                }
                height = owner._prepareHeaderCell(cell, indexPath);
            }
            return view_1.layout.toDeviceIndependentPixels(height);
        }
        if (indexPath.row === total - 1 && owner._getHasFooter()) {
            var height_1 = owner.getFooterHeight(indexPath.section);
            if (height_1 === undefined) {
                var template = owner._getFooterTemplate(indexPath.section);
                var cell = this._measureFooterMap.get(template.key);
                if (!cell) {
                    cell = tableView.dequeueReusableCellWithIdentifier("footer-" + template.key) || AccordionFooterCell.initWithEmptyBackground();
                    this._measureFooterMap.set(template.key, cell);
                }
                height_1 = owner._prepareFooterCell(cell, indexPath);
            }
            return view_1.layout.toDeviceIndependentPixels(height_1);
        }
        height = owner.getItemContentHeight(parseInt("" + (indexPath.section + 1) + indexPath.row));
        if (height === undefined) {
            var template = owner._getItemContentTemplate(indexPath.section, indexPath.row);
            var cell = this._measureItemContentMap.get(template.key);
            if (!cell) {
                cell = tableView.dequeueReusableCellWithIdentifier("item-content-" + template.key) || AccordionCell.initWithEmptyBackground();
                this._measureItemContentMap.set(template.key, cell);
            }
            height = owner._prepareCell(cell, indexPath);
        }
        return view_1.layout.toDeviceIndependentPixels(height);
    };
    UITableViewDelegateImpl.prototype.tableViewHeightForHeaderInSection = function (tableView, section) {
        var owner = this._owner.get();
        if (!owner) {
            return tableView.estimatedSectionHeaderHeight;
        }
        var height = owner.getItemHeaderHeight(section);
        if (height === undefined) {
            var template = owner._getItemHeaderTemplate(section);
            var header = this._measureItemHeaderMap.get(template.key);
            if (!header) {
                header = tableView.dequeueReusableHeaderFooterViewWithIdentifier("item-header-" + template.key) || AccordionHeader.initWithEmptyBackground();
                this._measureItemHeaderMap.set(template.key, header);
            }
            height = owner._prepareItemHeaderView(header, section);
        }
        return view_1.layout.toDeviceIndependentPixels(height);
    };
    UITableViewDelegateImpl.prototype.tableViewDidSelectRowAtIndexPath = function (tableView, indexPath) {
        var owner = this._owner.get();
        var data = owner._getChildData(indexPath.section, indexPath.row);
        var args = {
            eventName: accordion_common_1.AccordionBase.itemContentTapEvent,
            data: data,
            object: owner,
            childIndex: indexPath.row,
            index: indexPath.section,
            view: null,
            ios: null,
            android: undefined
        };
        owner.notify(args);
    };
    UITableViewDelegateImpl.prototype.tableViewViewForHeaderInSection = function (tableView, section) {
        var tapGesture = UITapGestureRecognizer.alloc().initWithTargetAction(AccordionHeaderTap.initWithOwner(this._owner), 'tap');
        var owner = this._owner.get();
        var header;
        if (owner) {
            var template = owner._getItemHeaderTemplate(section);
            header = (tableView.dequeueReusableHeaderFooterViewWithIdentifier("item-header-" + template.key) || AccordionHeader.initWithEmptyBackground());
            owner._prepareItemHeaderView(header, section);
            var cellView = header.view;
            if (cellView && cellView.isLayoutRequired) {
                var width = view_1.layout.getMeasureSpecSize(owner.widthMeasureSpec);
                var rowHeight = owner._effectiveRowHeight;
                var cellHeight = rowHeight > 0 ? rowHeight : owner.getItemHeaderHeight(section);
                view_1.View.layoutChild(owner, cellView, 0, 0, width, cellHeight ? cellHeight : 0);
            }
        }
        else {
            header = AccordionHeader.initWithEmptyBackground();
        }
        header.addGestureRecognizer(tapGesture);
        header.tag = section;
        return header;
    };
    UITableViewDelegateImpl.ObjCProtocols = [UITableViewDelegate];
    return UITableViewDelegateImpl;
}(NSObject));
exports.UITableViewDelegateImpl = UITableViewDelegateImpl;
//# sourceMappingURL=accordion.ios.js.map